#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_mixer.h"
#include "SDL_ttf.h"
#include "SDL_image.h"

#include "BList.h"
#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "tiles.h"
#include "samples.h"
#include "interfaces.h"

#include "animscript.h"
#include "actionscript.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

MOG2::MOG2(void)
{
	game_action=new ActionSCRIPT();
	n_app_states=1;
	s_app_action=new ActionSCRIPT *[1];
	s_app_action[0]=new ActionSCRIPT();

	/* Initialize application variables: */ 
	{
		VARIABLE *v;

		v=new VARIABLE("screen_x",640);
		application_variables.Add(v);
		v=new VARIABLE("screen_y",480);
		application_variables.Add(v);
		v=new VARIABLE("fps",25);
		application_variables.Add(v);
		v=new VARIABLE("hw_render",0);
		application_variables.Add(v);
		v=new VARIABLE("player_plane",0);
		application_variables.Add(v);
		v=new VARIABLE("sfx_atenuation_min",-1);
		application_variables.Add(v);
		v=new VARIABLE("sfx_atenuation_max",-1);
		application_variables.Add(v);
		v=new VARIABLE("state",0);
		application_variables.Add(v);
	}

} /* MOG2::MOG2 */ 


MOG2::~MOG2(void)
{
	int i;

	FONTCONTAINER *fc;

	while(!fonts.EmptyP()) {
		fc=fonts.ExtractIni();
		delete fc;
	} /* while */ 


	if (game_action!=0) delete game_action;
	game_action=0;

	for(i=0;i<n_app_states;i++) {
		delete s_app_action[i];
		s_app_action[i]=0;
	} /* for */ 
	delete []s_app_action;
	s_app_action=0;
} /* MOG2::~MOG2 */ 


bool MOG2::save(FILE *fp)
{
	List<VARIABLE> l;
	VARIABLE *v;
	List<MAP> l2;
	MAP *m;
	List<TILE_SOURCE> l3;
	TILE_SOURCE *b;
	List<TILE> l4;
	TILE *t;
	List<OBJECT> l5;
	OBJECT *o;
	List<SAMPLE> l6;
	SAMPLE *s;
	List<MOG2_INTERFACE> l7;
	MOG2_INTERFACE *i_f;
	int i;

	{
		unsigned int nt=0;
		unsigned int no=0;
		unsigned int nr=0;
		unsigned int nm=0;

		for(i=0;i<TILECATEGORIES;i++) nt+=tiles[i].Length();
		no=objects.Length();
		l2.Instance(maps);
		l2.Rewind();
		while(l2.Iterate(m)) nr+=m->rooms.Length();
		nm=maps.Length();

		nt=nt%256;
		no=no%256;
		nr=nr%256;
		nm=nm%256;

		fprintf(fp,"ID_HIGH %u\n",(unsigned int)(nt+(no*256)+(nr*256*256)+(nm*256*256*256)));
		fprintf(fp,"ID_LOW %u\n",(unsigned int)SDL_GetTicks());
	}

	fprintf(fp,"TILE_SOURCES %i\n",tile_sources.Length());
	l3.Instance(tile_sources);
	l3.Rewind();
	while(l3.Iterate(b)) b->save(fp);

	for(i=0;i<TILECATEGORIES;i++) {
		fprintf(fp,"TILES %i\n",tiles[i].Length());
		l4.Instance(tiles[i]);
		l4.Rewind();
		while(l4.Iterate(t)) t->save(fp);
	} /* for */ 

	fprintf(fp,"SAMPLES %i\n",samples.Length());
	l6.Instance(samples);
	l6.Rewind();
	while(l6.Iterate(s)) s->save(fp);

	fprintf(fp,"OBJECTS %i\n",objects.Length());
	l5.Instance(objects);
	l5.Rewind();
	while(l5.Iterate(o)) o->save(fp,this);
	
	fprintf(fp,"GVARIABLES %i\n",game_variables.Length());
	l.Instance(game_variables);
	l.Rewind();
	while(l.Iterate(v)) v->save(fp);

	fprintf(fp,"MAPS %i\n",maps.Length());
	l2.Instance(maps);
	l2.Rewind();
	while(l2.Iterate(m)) m->save(fp,this);

	fprintf(fp,"INTERFACES %i\n",interfaces.Length());
	l7.Instance(interfaces);
	l7.Rewind();
	while(l7.Iterate(i_f)) i_f->save(fp,this);
		
	game_action->save(fp);

	fprintf(fp,"AVARIABLES %i\n",application_variables.Length());
	l.Instance(application_variables);
	l.Rewind();
	while(l.Iterate(v)) v->save(fp);

	fprintf(fp,"%i\n",n_app_states);
	for(i=0;i<n_app_states;i++) s_app_action[i]->save(fp);

	return true;
} /* MOG2::save */ 


bool MOG2::load(FILE *fp)
{
	char tmp[80];
	int i,j,n;
	List<TILE_SOURCE> l;
	TILE_SOURCE *t;

	if (2!=fscanf(fp,"%s %u",tmp,&ID_high)) return false;
	if (2!=fscanf(fp,"%s %u",tmp,&ID_low)) return false;

	if (2!=fscanf(fp,"%s %i",tmp,&n)) return false;
	tile_sources.Delete();
	for(i=0;i<n;i++) {
		TILE_SOURCE *b;
		bool found;

		b=new TILE_SOURCE();
		b->load(fp);

		found=false;
		l.Instance(tile_sources);
		l.Rewind();
		while(l.Iterate(t) && !found) if (t->cmp(b)) found=true;
		if (!found) tile_sources.Add(b);
			   else delete b;
	} /* for */ 

	for(j=0;j<TILECATEGORIES;j++) {
		if (2!=fscanf(fp,"%s %i",tmp,&n)) return false;
		tiles[j].Delete();
		for(i=0;i<n;i++) {
			TILE *t;
			t=new TILE();
			t->load(fp,this);
			tiles[j].Add(t);
		} /* for */ 
	} /* for */ 

	if (2!=fscanf(fp,"%s %i",tmp,&n)) return false;
	samples.Delete();
	for(i=0;i<n;i++) {
		SAMPLE *s;
		s=new SAMPLE();
		s->load(fp);
		samples.Add(s);
	} /* for */ 

	if (2!=fscanf(fp,"%s %i",tmp,&n)) return false;
	objects.Delete();
	for(i=0;i<n;i++) {
		OBJECT *o;
		o=new OBJECT();
		if (!o->load(fp,this)) return false;
		objects.Add(o);
	} /* for */ 

	if (2!=fscanf(fp,"%s %i",tmp,&n)) return false;
	game_variables.Delete();
	for(i=0;i<n;i++) {
		VARIABLE *v;
		v=new VARIABLE();
		if (!v->load(fp)) return false;
		game_variables.Add(v);
	} /* for */ 

	if (2!=fscanf(fp,"%s %i",tmp,&n)) return false;
	maps.Delete();
	for(i=0;i<n;i++) {
		MAP *m;
		m=new MAP(this);
		if (!m->load(fp,this)) return false;
		maps.Add(m);
	} /* for */ 

	if (2!=fscanf(fp,"%s %i",tmp,&n)) return false;
	interfaces.Delete();
	for(i=0;i<n;i++) {
		MOG2_INTERFACE *i_f;
		i_f=new MOG2_INTERFACE();
		if (!i_f->load(fp,this)) return false;
		interfaces.Add(i_f);
	} /* for */ 

	if (game_action!=0) delete game_action;
	game_action=new ActionSCRIPT();
	game_action->load(fp,this);

	if (2!=fscanf(fp,"%s %i",tmp,&n)) return false;
	application_variables.Delete();
	for(i=0;i<n;i++) {
		VARIABLE *v;
		v=new VARIABLE();
		if (!v->load(fp)) return false;
		application_variables.Add(v);
	} /* for */ 

	for(i=0;i<n_app_states;i++) {
		delete s_app_action[i];
		s_app_action[i]=0;
	} /* for */ 
	delete []s_app_action;
	s_app_action=0;

	if (1!=fscanf(fp,"%i",&n_app_states)) return false;
	s_app_action=new ActionSCRIPT *[n_app_states];
	for(i=0;i<n_app_states;i++) {
		s_app_action[i]=new ActionSCRIPT();
		s_app_action[i]->load(fp,this);
	} /* for */ 

	objects.Sort(&object_order);

	return true;
} /* MOG2::load */ 


OBJECT *MOG2::find_object(char *name)
{
	List<OBJECT> l;
	OBJECT *o;

	l.Instance(objects);
	l.Rewind();
	while(l.Iterate(o)) {
		if (o->cmp(name)) {
			return o;
		} /* if */ 
	} /* while */ 

	return 0;
} /* MOG2::find_object */ 


FONTCONTAINER *MOG2::get_font(char *name,int size)
{
	List<FONTCONTAINER> l;
	FONTCONTAINER *fc;

	if (size==0) size=8;

	l.Instance(fonts);
	l.Rewind();
	while(l.Iterate(fc)) {
		if (strcmp(fc->name,name)==0 &&
			fc->size==size) {
			return fc;
		} /* if */ 
	} /* while */ 

	fc=new FONTCONTAINER(name,size);
	if (fc->bitmap_font!=0 || fc->ttf_font!=0) {
		fonts.Add(fc);
		return fc;
	} /* if */ 

	delete fc;
	return 0;
} /* MOG2::get_font */ 
